@@ -4,6 +4,7 @@ knitr::opts_knit$set(global.par = TRUE)
 
 ## ---- echo = FALSE------------------------------------------------------------
 set.seed(5)
+oldmar <- par()$mar
 par(mar = c(0, 0, 0, 0) + 0.1)
 
 ## ----setup--------------------------------------------------------------------
@@ -117,3 +118,6 @@ fastball(mat)
 mat <- rbind(c(1,0,0), c(0,0,1), c(0,1,1))
 bicm(mat)
 
+## ---- echo = FALSE------------------------------------------------------------
+par(mar = oldmar)
+
